import QtQuick 2.2
import QtQuick.Layouts 1.11
import QtQuick.Controls 2.15 as QQC2

import org.kde.kcm 1.3 as KCM
import org.kde.kirigami 2.19 as Kirigami
import org.kde.kirigamiaddons.treeview 1.0 as TreeView
import org.kde.kirigamiaddons.labs.mobileform 0.1 as MobileForm

import IntegrityFileSystemModel 1.0
import ServiceHelper 1.0
import FailureScriptsHelper 1.0
import IntegrityControlScriptCaller 1.0

KCM.ScrollViewKCM {
    id: root

    Kirigami.Theme.colorSet: Kirigami.Theme.Window

    title: i18nc("@title", "Integrity control")

    function save() {
        kcm.saveConfig();
        integrityScriptsLayout.enabled = false;
        if (!integrityScriptCaller.initializeAfickDatabase()) {
            saveChangesErrorMessage.visible = true;
            if (kcm.rollbackConfig()) {
                log.append(i18n("Initialize Afick database was successful"));
            } else {
                log.append(i18n("Afick configuration has rolled back"));
                integrityScriptCaller.initializeAfickDatabase();
            }
            kcm.loadConfig();
        } else {
            saveChangesErrorMessage.visible = false;
        }

        integrityScriptsLayout.enabled = true;
        showLogButton.visible = true;
    }

    actions {
        contextualActions: [
            Kirigami.Action {
                text: i18nc("@action:button", "Reset")
                enabled: kcm.needsSave
                onTriggered: kcm.loadConfig()
            },

            Kirigami.Action {
                text: i18nc("@action:button", "Save")
                enabled: kcm.needsSave
                onTriggered: root.save()
            }
        ]
    }

    view: Flickable {
        id: flickable
        contentWidth: mainLayout.width
        contentHeight: mainLayout.height
        clip: true

        ColumnLayout {
            id: mainLayout
            width: flickable.width

            ServiceHelper {
                id: serviceHelper
            }

            FailureScriptsHelper {
                id: failureHelper
            }

            IntegrityControlScriptCaller {
                id: integrityScriptCaller

                onInitializeError: log.append(error);
                onIntegrityError: log.append(error);
            }

            Kirigami.InlineMessage {
                id: saveChangesErrorMessage
                Kirigami.FormData.isSection: true
                Layout.fillWidth: true
                type: Kirigami.MessageType.Error
                text: i18n("An error occurred while saving configuration changes")
                visible: false
            }

            MobileForm.FormCard {
                Layout.fillWidth: true

                contentItem: ColumnLayout {
                    spacing: 0

                    MobileForm.FormCardHeader {
                        title: i18n("Integrity control service")
                    }

                    MobileForm.FormSwitchDelegate {
                        id: enableSwitch
                        text: i18n("Enable integity control at system startup")
                        checked: serviceHelper.isEnable
                        onClicked: serviceHelper.isEnable = checked
                    }

                    ColumnLayout {
                        visible: serviceHelper.isEnable
                        spacing: 0
                        MobileForm.FormCardHeader {
                            title: i18n("Integriry check failure scripts")
                        }

                        Repeater {
                            model: failureHelper.scripts
                            delegate: MobileForm.FormSwitchDelegate {
                                id: scriptEnableSwitch
                                text: modelData
                                checked: modelData.endsWith(".sh")
                                onClicked: failureHelper.enableScript(modelData, checked)

                                Connections {
                                    target: failureHelper

                                    function onScriptEnableChanged(name, enable) {
                                        if (scriptEnableSwitch.text === name && scriptEnableSwitch.checked !== enable)
                                            scriptEnableSwitch.checked = enable;
                                    }
                                }
                            }
                        }
                        QQC2.Label { // MobileForm.FormTextDelegate has Text.Wrap wrapMode, better Text.WordWrap
                            Layout.fillWidth: true
                            padding: Kirigami.Units.gridUnit
                            visible: failureHelper.scripts.length === 0
                            opacity: 0.8
                            wrapMode: Text.WordWrap
                            text: i18n("No scripts have been set to handle the integrity check error.\nYou can add your handler script to the %1 directory",
                                       failureHelper.scriptsPath())
                        }
                    }

                    MobileForm.FormCardHeader {
                        title: i18n("Afick controls")
                    }

                    ColumnLayout {
                        id: integrityScriptsLayout
                        spacing: 0
                        MobileForm.FormButtonDelegate {
                            text: i18n("Initialize Afick database")
                            description: i18n("Set the current state of the system as a reference")
                            onClicked: {
                                integrityScriptsLayout.enabled = false;
                                if (integrityScriptCaller.initializeAfickDatabase()) {
                                    log.append(i18n("Initialize Afick database was successful"));
                                    icon.name = "dialog-ok";
                                }
                                else
                                {
                                    icon.name = "dialog-cancel";
                                }

                                integrityScriptsLayout.enabled = true;
                                showLogButton.visible = true;
                            }
                        }
                        MobileForm.FormButtonDelegate {
                            text: i18n("Integrity check")
                            description: i18n("Compare the file system with the database")

                            onClicked: {
                                integrityScriptsLayout.enabled = false;
                                if (integrityScriptCaller.checkIntegrity()) {
                                    log.append(i18n("Integrity check was successful"));
                                    icon.name = "dialog-ok";
                                }
                                else
                                {
                                    icon.name = "dialog-cancel";
                                }

                                integrityScriptsLayout.enabled = true;
                                showLogButton.visible = true;
                            }
                        }
                        MobileForm.AbstractFormDelegate {
                            Layout.fillWidth: true
                            background: Item {}
                            clip: true
                            leftPadding: 0
                            topPadding:  0
                            bottomPadding: 0
                            rightPadding: 0

                            contentItem: ColumnLayout {
                                spacing: 0

                                MobileForm.FormButtonDelegate {
                                    id: showLogButton
                                    text: i18n("Show log")
                                    visible: false
                                    onClicked: {
                                        logWrapper.visible = !logWrapper.visible;
                                        text = logWrapper.visible ? i18n("Hide log") : i18n("Show log");
                                    }

                                    background: Rectangle {
                                        id: showLogButtonBackground
                                        color: "transparent"
                                        SequentialAnimation {
                                            id: focusLogAnim
                                            loops: 3
                                            PropertyAnimation {
                                                target: showLogButtonBackground
                                                property: "color"
                                                to: Qt.rgba(Kirigami.Theme.textColor.r, Kirigami.Theme.textColor.g, Kirigami.Theme.textColor.b, 0.2)
                                                duration: Kirigami.Units.veryLongDuration

                                            }
                                            PropertyAnimation {
                                                target: showLogButtonBackground
                                                property: "color"
                                                to: "transparent"
                                                duration: Kirigami.Units.veryLongDuration
                                            }
                                        }
                                    }
                                }
                                MobileForm.FormDelegateSeparator {}
                                Flickable {
                                    id: logWrapper
                                    flickableDirection: Flickable.VerticalFlick
                                    implicitHeight: 200
                                    implicitWidth: parent.width
                                    visible: false

                                    QQC2.TextArea.flickable: QQC2.TextArea {
                                        id: log
                                        enabled: false
                                        background: Item {}

                                        implicitWidth: parent.width
                                        wrapMode: QQC2.TextArea.WordWrap


                                        // scroll log to bottom
                                        onTextChanged: {
                                            cursorPosition = length-1;
                                            if (!logWrapper.visible) // notify that we have new notifications
                                                focusLogAnim.restart();
                                        }
                                    }

                                    QQC2.ScrollBar.vertical: QQC2.ScrollBar {}
                                }
                            }
                        }
                    }

                }
            }

            MobileForm.FormCard {
                Layout.fillWidth: true

                contentItem: ColumnLayout {
                    spacing: 0

                    MobileForm.FormCardHeader {
                        title: i18n("Rule and Alias Settings")
                    }

                    MobileForm.FormButtonDelegate {
                        text: i18n("Alias overview")
                        description: i18n("Add, delete, or edit an alias")
                        onClicked: {
                            kcm.push("integrityAliasEditor.qml", {mainPage: root});
                        }
                    }

                    MobileForm.FormDelegateSeparator {}

                    MobileForm.AbstractFormDelegate {
                        background: Item {}
                        clip: true
                        leftPadding: 0
                        topPadding: 0
                        bottomPadding: 0
                        rightPadding: 0
                        contentItem: ColumnLayout {
                            id: layout

                            MobileForm.FormTextDelegate {
                                text: i18n("Directory and file settings")
                            }

                            TreeView.TreeListView {
                                id: view
                                model: kcm.integrityFileSystemModel
                                implicitHeight: contentHeight
                                interactive: false // disable scrolling
                                Layout.fillHeight: true
                                Layout.fillWidth: true

                                clip: true

                                delegate: IntegrityBasicTreeItem {
                                    id: treeItem
                                    label: model.fileName
                                    iconSource: model.fileIcon
                                    integrityPrefix: model.integrityPrefix
                                    integrityAttributes: model.integrityAttributes
                                    integrityElementEdited: model.integrityElementChanged

                                    onClicked: kcm.integrityFileSystemModel.loadDirectory(model.filePath)

                                    onEditItemClicked: {
                                        kcm.integrityFileSystemModel.setCurrentRule(model.filePath);
                                        kcm.push("integrityControlElement.qml", {mainPage: root});
                                        // save select when we show element
                                        treeItem.forceActiveFocus();
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        Connections {
            target: serviceHelper

            function onEnableChanged(val) {
                if (enableSwitch.checked !== val)
                    enableSwitch.checked = val;
            }
        }
    }

    Component.onCompleted: {
        kcm.initAfickSync();
    }
}
